apiVersion: v1
kind: Secret
metadata:
  namespace: "{{ ns }}"
  name: "{{ name }}"
  labels:
    app.kubernetes.io/name: "{{ name }}"
type: Opaque
data:
  PASSWORD: "{{ get_secret('/application', 24) | b64encode }}"
